﻿using System;

using Smdn.Protocols.Imap4.Protocol;

namespace Smdn.Protocols.Imap4.Transaction.Transactions {
  internal abstract class TransactionBase : ImapTransaction {
    protected TransactionBase(ImapConnection connection, ImapRequest request)
      : base(connection, request)
    {
    }

    protected bool Aborted {
      get { return aborted; }
    }

    public override void Abort()
    {
      aborted = true;

      base.Abort();
    }

    protected bool Receive(out ImapProtocolResponse response)
    {
      response = null;

      if (aborted)
        return false;

      response = Connection.TryReceiveResponse();

      if (response is ImapStatusResponse) {
        var status = response as ImapStatusResponse;
        string message;
        if (ImapStatusResponseConverter.FromParse(status, out message)) {
          // PARSE
          Trace.Info("[PARSE] {0}", message);
        }
        else if (ImapStatusResponseConverter.FromAlert(status, out message)) {
          // ALERT
          Trace.Info("[ALERT] {0}", message);
          Console.Error.WriteLine("[ALERT] {0}", message);
        }
      }

      return (response != null);
    }

    protected string Send(string command, params string[] parameters)
    {
      if (aborted)
        return string.Empty;

      var comm = Connection.CreateCommand(command, parameters);

      Send(comm);

      return comm.Tag;
    }

    protected string SendContinue(params string[] parameters)
    {
      if (aborted)
        return string.Empty;

      var comm = Connection.CreateContinuingCommand(parameters);

      Send(comm);

      return comm.Tag;
    }

    private void Send (ImapProtocolCommand command)
    {
      if (aborted)
        return;

      Connection.SendCommand(command);
    }

    protected void Conclude(ImapResponse response)
    {
      if (aborted)
        return;

      this.Response = response;

      Transit(null);
    }

    protected void ConcludeError(ImapStatusResponse response)
    {
      if (aborted)
        return;

      if (response.Status ==ImapProtocolStatus.No) {
        Conclude(new ImapResponse(ImapTransactionStatus.Forbidden, response.Text));
      }
      else if (response.Status == ImapProtocolStatus.Bad) {
        Conclude(new ImapResponse(ImapTransactionStatus.InternalServerError, response.Text));
      }
      else if (response.Status == ImapProtocolStatus.Bye) {
        Conclude(new ImapResponse(ImapTransactionStatus.Disconnecting, response.Text));
      }
      else {
        Conclude(new ImapResponse(ImapTransactionStatus.BadRequest, response.Text));
      }
    }

    private bool aborted = false;
  }
}
