﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Text;

namespace Smdn.Protocols.Pop3 {
  // RFC 2384 POP URL Scheme
  // http://tools.ietf.org/html/rfc2384
  
  //    A POP URL is of the general form:
  //        pop://<user>;auth=<auth>@<host>:<port>

  public class PopUri : Uri {
    public const string UriSchemePop = "pop";
    internal const int DefaultPort = 110;

    /// <summary>
    /// returns username without authentication type
    /// </summary>
    public string UserName {
      get
      {
        if (UserInfo.Contains(";"))
          return UserInfo.Substring(0, UserInfo.IndexOf(";"));
        else
          return UserInfo;
      }
    }

    /// <summary>
    /// returns username@host
    /// </summary>
    public string UserAtHost {
      get { return string.Format("{0}@{1}", UserName, Host); }
    }

    /// <summary>
    /// returns authentication type
    /// </summary>
    public PopAuthenticationType Authentication {
      get
      {
        var index = UserInfo.ToUpper().IndexOf(";AUTH=");

        if (index == -1)
          return PopAuthenticationType.None;
        else
          return PopConverter.ToAuthType(UserInfo.Substring(index + 6));
      }
    }

    internal PopUriForm Form {
      get
      {
        return PopUriForm.Server;
      }
    }

    public PopUri(Uri uri)
      : this(uri.ToString())
    {
    }

    public PopUri(string uriString)
      : base(uriString)
    {
      if (Scheme != UriSchemePop)
        throw new ArgumentException(string.Format("scheme must be {0}", UriSchemePop));
    }
  }
}
