using System;
using System.Collections.Generic;

namespace Smdn.Protocols.Pop3.Transaction {
  // Layers:
  //   Client.PopClient
  //   Client.PopClientAgent
  //   Session.PopSessionAgent
  //   Session.PopSession
  //   Transaction.PopTransactionAgent
  // * Transaction.PopTransaction/Transaction.PopRequest/Transaction.PopResponse
  //   Protocol.PopConnection/Protocol.PopProtocolCommand/Protocol.PopProtocolResponse
  //   (POP3)

  public class PopRequest {
    public PopTransactionMethod Method {
      get { return method; }
    }

    public Dictionary<string, string> Parameters {
      get { return parameters; }
    }

    public int Timeout {
      get { return timeout; }
    }

    internal DateTime RequestedTime {
      get; set;
    }

    internal PopRequest(PopTransactionMethod method, int timeout)
      : this(method, timeout, null)
    {
    }

    internal PopRequest(PopTransactionMethod method, int timeout, IDictionary<string, string> parameters)
    {
      this.method = method;
      this.timeout = timeout;

      if (parameters != null) {
        foreach (var pair in parameters) {
          this.parameters.Add(pair.Key, pair.Value);
        }
      }
    }

    public override string ToString()
    {
      return string.Format("{{Method={0}, Parameters={1}}}", method, parameters.Count);
    }

    private readonly Dictionary<string, string> parameters = new Dictionary<string, string>();
    private readonly PopTransactionMethod method;
    private readonly int timeout;
  }
}
