using System;
using System.Text;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal class TopTransaction : TransactionBase {

    // ProcessTopIncapable
    //   x Conclude
    // ProcessParameterNotSetted
    //   x Conclude
    // ProcessTop
    //   ->WaitForTop
    // WaitForTop
    //   x Conclude
    // RetrieveTop
    //   x Conclude

    public TopTransaction(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected override ProcessTransactionDelegate Reset()
    {
      if ((Connection.ServerInfo.Capability.KnownCapabilities & PopKnownCapabilities.Capabilities) != 0 &&
          (Connection.ServerInfo.Capability.KnownCapabilities & PopKnownCapabilities.Top) == 0)
        return ProcessTopIncapable;

      if (!Request.Parameters.ContainsKey("Message-Number"))
        return ProcessParameterNotSetted;

      if (!Request.Parameters.ContainsKey("Lines"))
        return ProcessParameterNotSetted;

      messageNumber = Request.Parameters["Message-Number"];
      lines = Request.Parameters["Lines"];

      return ProcessTop;
    }

    private void ProcessTopIncapable()
    {
      Conclude(new PopResponse(PopTransactionStatus.Incapable, "TOP incapable"));
    }

    private void ProcessParameterNotSetted()
    {
      Conclude(new PopResponse(PopTransactionStatus.BadRequest, "parameter 'Message-Number' and 'Lines' must be setted"));
    }

    private void ProcessTop()
    {
      Send("TOP", messageNumber, lines);

      Transit(WaitForTop);
    }

    private void WaitForTop()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopStatusResponse) {
        var status = response as PopStatusResponse;

        if (status.Status == PopProtocolStatus.Positive) {
          retrieved = new StringBuilder();
          Transit(RetrieveTop);
        }
        else {
          ConcludeError(status);
        }
      }
    }

    private void RetrieveTop()
    {
      PopProtocolResponse response;

      if (!Receive(out response))
        return;

      if (response is PopFollowingResponse) {
        var following = response as PopFollowingResponse;

        if (following.Text.StartsWith("."))
          // remove byte-stuff
          retrieved.AppendLine(following.Text.Substring(1));
        else
          retrieved.AppendLine(following.Text);
      }
      else if (response is PopTerminationResponse) {
        Conclude(new PopMessageResponse(PopTransactionStatus.Ok,
                                         new PopMessage(int.Parse(messageNumber), retrieved.ToString()),
                                         "retrieved a message"));
      }
    }

    private string messageNumber;
    private string lines;
    private StringBuilder retrieved = null;
  }
}
