﻿using System;

using Smdn.Protocols.Pop3.Protocol;

namespace Smdn.Protocols.Pop3.Transaction.Transactions {
  internal abstract class TransactionBase :PopTransaction {
    protected TransactionBase(PopConnection connection, PopRequest request)
      : base(connection, request)
    {
    }

    protected bool Aborted {
      get { return aborted; }
    }

    public override void Abort()
    {
      aborted = true;

      base.Abort();
    }

    protected bool Receive(out PopProtocolResponse response)
    {
      response = null;

      if (aborted)
        return false;

      response = Connection.TryReceiveResponse();

      return (response != null);
    }

    protected void Send(string command, params string[] parameters)
    {
      if (aborted)
        return;

      var comm = Connection.CreateCommand(command, parameters);

      Connection.SendCommand(comm);
    }

    protected void Conclude(PopResponse response)
    {
      if (aborted)
        return;

      this.Response = response;

      Transit(null);
    }

    protected void ConcludeError(PopStatusResponse status)
    {
      if (aborted)
        return;

      Conclude(new PopResponse(PopTransactionStatus.Forbidden, status.Message));
    }

    private bool aborted = false;
  }
}
